@@ -23,7 +23,6 @@
 	    _b_vector-length
 	    theme-string-match
 	    integer->real
-	    real->integer
 	    theme-round
 	    theme-truncate
 	    theme-floor
@@ -45,6 +44,9 @@
 	    theme-current-output-port
 	    theme-current-error-port
 	    theme-current-input-port
+	    theme-standard-output-port
+	    theme-standard-error-port
+	    theme-standard-input-port
 	    theme-call-with-input-string
 	    theme-call-with-output-string
 	    theme-prim-console-display
@@ -175,40 +177,41 @@
 (define (theme-string-match str-pattern str-source)
   (let ((match-result (string-match str-pattern str-source)))
     (if (regexp-match? match-result)
-	(list
-	 (match:substring match-result)
-	 (match:start match-result)
-	 (match:end match-result))
+	(let ((str-sub (match:substring match-result))
+	      (i-start (match:start match-result))
+	      (i-end (match:end match-result)))
+	  (if (and (string? str-sub)
+		   (is-integer? i-start)
+		   (is-integer? i-end))
+	      (list str-sub i-start i-end)
+	      (raise (make-theme-d-condition
+		      'theme-string-match:internal-error
+		      (list (cons 'str-pattern str-pattern)
+			    (cons 'str-source str-source))))))
 	(begin
 	  (assert (eq? match-result #f))
 	  '()))))
 
 
 (define (integer->real n)
-  (assert (integer? n))
+  (assert (is-integer? n))
   (exact->inexact n))
 
 
-(define (real->integer r)
-  (if (integer-valued? r)
-      (inexact->exact r)
-      (raise-simple 'numeric-type-mismatch)))
-
-
 (define (theme-round r)
-  (inexact->exact (r-round r)))
+  (real->integer (r-round r)))
 
 
 (define (theme-truncate r)
-  (inexact->exact (r-truncate r)))
+  (real->integer (r-truncate r)))
 
 
 (define (theme-floor r)
-  (inexact->exact (r-floor r)))
+  (real->integer (r-floor r)))
 
 
 (define (theme-ceiling r)
-  (inexact->exact (r-ceiling r)))
+  (real->integer (r-ceiling r)))
 
 
 (define xor
@@ -267,8 +270,13 @@
 
 
 (define (real-to-rational-impl r)
-  (let ((nr-result (inexact->exact r)))
-    (cons (numerator nr-result) (denominator nr-result))))
+  (let* ((nr-result (inexact->exact r))
+	 (i-numer (numerator nr-result))
+	 (i-denom (denominator nr-result)))
+    (if (and (is-integer? i-numer) (is-integer? i-denom))
+	(cons (numerator nr-result) (denominator nr-result))
+	(raise (make-theme-d-condition 'real-to-rational-impl:internal-error
+				       (list (cons 'r-value r)))))))
 
 
 ;; *** (standard-library text-file-io) ***
@@ -410,6 +418,7 @@
   	  (check-read-data? data)
   	  data))))
 
+
 (define (theme-char-ready? ip)
   (guard (exc (else
   	       (raise (make-file-exception
@@ -423,21 +432,51 @@
   (guard (exc (else
   	       (raise (make-file-exception
   		       'current-output-port:runtime-error ""))))
-  	 (current-output-port)))
+	 (let ((port (current-output-port)))
+	   (if (output-port? port)
+	       port
+	       (raise-simple 'theme-current-output-port:internal-error)))))
 
 
 (define (theme-current-error-port)
   (guard (exc (else
   	       (raise (make-file-exception
   		       'current-error-port:runtime-error ""))))
-  	 (current-error-port)))
+	 (let ((port (current-error-port)))
+	   (if (output-port? port)
+	       port
+	       (raise-simple 'theme-current-error-port:internal-error)))))
 
 
 (define (theme-current-input-port)
   (guard (exc (else
   	       (raise (make-file-exception
   		       'current-input-port:runtime-error ""))))
-  	 (current-input-port)))
+	 (let ((port (current-input-port)))
+	   (if (input-port? port)
+	       port
+	       (raise-simple 'theme-current-input-port:internal-error)))))
+
+
+(define (theme-standard-output-port)
+  (let ((port (standard-output-port)))
+    (if (output-port? port)
+	port
+	(raise-simple 'theme-standard-output-port:internal-error))))
+
+
+(define (theme-standard-error-port)
+  (let ((port (standard-error-port)))
+    (if (output-port? port)
+	port
+	(raise-simple 'theme-standard-error-port:internal-error))))
+
+  
+(define (theme-standard-input-port)
+  (let ((port (standard-input-port)))
+    (if (input-port? port)
+	port
+	(raise-simple 'theme-standard-input-port:internal-error))))
 
 
 (define (theme-call-with-input-string str proc)
@@ -532,7 +571,10 @@
 (define (reverse-get-goops-class clas)
   (let ((o (reverse-search-goops-class clas)))
     (if (not (eq? o #f))
-	o
+	(if (is-a? o <class>)
+	    o
+	    (raise (make-theme-d-condition 'internal-error-with-goops-class
+					   (list (cons 'cl clas)))))
 	(raise (make-theme-d-condition 'undefined-goops-class
 				       (list (cons 'cl clas)))))))
 
@@ -637,28 +679,33 @@
 
 (define (theme-string->integer str)
   (let ((nr (string->number str)))
-    (if (integer? nr)
+    (if (is-integer? nr)
 	nr
 	(raise (make-theme-d-condition 'string-to-integer-conv-error
 				       (list (cons 'str str)))))))
 
 
 (define (theme-string->real str)
-  (let ((nr (string->number str)))
-    (cond
-     ((integer? nr) (exact->inexact nr))
-     ((rational? nr) (exact->inexact nr))
-     ((and (real? nr) (not (exact? nr))) nr)
-     (else
-      (raise (make-theme-d-condition 'string-to-real-conv-error
-				     (list (cons 'str str))))))))
-
+  (let* ((nr (string->number str))
+	 (r-result
+	  (cond
+	   ((is-integer? nr) (exact->inexact nr))
+	   ((rational? nr) (exact->inexact nr))
+	   ((is-real? nr) nr)
+	   (else
+	    (raise (make-theme-d-condition 'string-to-real-conv-error
+					   (list (cons 'str str))))))))
+    (if (is-real? r-result)
+	r-result
+	(raise (make-theme-d-condition 'theme-string->real:internal-error
+				       (list (cons 'str str)))))))
+    
 
 (define (theme-string->real-number str)
   (let ((nr (string->number str)))
     (cond
-     ((integer? nr) nr)
-     ((and (real? nr) (not (exact? nr))) nr)
+     ((is-integer? nr) nr)
+     ((is-real? nr) nr)
      ((rational? nr) (cons (numerator nr) (denominator nr)))
      (else
       (raise (make-theme-d-condition 'string-to-real-conv-error
