@@ -332,7 +332,7 @@ See `run-hooks'."
 	       "guard" "guard-general"
 	       "guard-nonpure" "guard-general-nonpure"
 	       "guard-without-result" "guard-general-without-result"
-	       "if" "iterate-list"
+	       "if" "iterate-list" "iterate-list-pure"
 	       "import" "import-and-reexport"
 	       "join-tuple-types"
 	       "lambda"
@@ -348,6 +348,7 @@ See `run-hooks'."
 	       "or"
 	       "param-lambda"
 	       "param-lambda-automatic"
+	       "param-method-dispatch"
 	       "param-prim-proc"
 	       "param-proc-cond-appl"
 	       "param-proc-instance" "param-proc-dispatch"
@@ -532,6 +533,7 @@ See `run-hooks'."
 (put 'syntax-rules 'theme-d-indent-function 1)
 (put 'until 'theme-d-indent-function 1)
 (put 'iterate-list 'theme-d-indent-function 1)
+(put 'iterate-list-pure 'theme-d-indent-function 1)
 
 ;; (put 'let-values 'theme-d-indent-function 1) ; SRFI 11
 ;; (put 'let*-values 'theme-d-indent-function 1) ; SRFI 11
